Ext.namespace("dSS", "dSS.config", "dSS.devprops");

// form *must* react to the submit event and fire a formsubmitted
// event in response; it *must* also fire a formloaded event once all d
// has been received
//
// parameters: dSUID
dSS.devprops.OutputBlinkConfigForm = Ext.extend(Ext.form.FormPanel, {
    constructor: function(config) {
        this.device= config.device;

        this.dSUID = this.device.get('dsuid');

        this.addEvents('formsubmitted', 'formloaded', 'formchanged');

        this.repetitions = new Ext.Panel(
        {
            border: false,
            height: 35,
            layout: 'hbox',
            items:
            [
                {
                    xtype: 'label',
                    text: dSS._("Repetitions") + ":",
                    width: 173,
                    style: { marginTop: '5px' }
                },
                {
                    xtype: 'spinnerfield',
                    width: 50,
                    ref: 'count',
                    minValue: 0,
                    maxValue: 255,
                    allowDecimals: false,
                    defaultValue: 1, // default value as defined by spec
                    allowEmpty: false
                }
            ]
        });

        this.onDelay = new Ext.Panel(
        {
            border: false,
            height: 35,
            layout: 'hbox',
            items:
            [
                {
                    xtype: 'label',
                    text: dSS._("Turn-on delay") + ":",
                    width: 173,
                    style: { marginTop: '5px' }
                },
                {
                    xtype: 'spinnerfield',
                    width: 50,
                    ref: 'seconds',
                    minValue: 0.0,
                    maxValue: 8.4,
                    allowDecimals: true,
                    defaultValue: 0,
                    allowEmpty: false
                },
                {
                    xtype: 'box',
                    width: 5
                },
                {
                    xtype: 'label',
                    text: dSS._("s"),
                    style: { marginTop: '5px' }
                }
            ]
        });

        this.offDelay = new Ext.Panel(
        {
            border: false,
            height: 25,
            layout: 'hbox',
            items:
            [
                {
                    xtype: 'label',
                    text: dSS._("Turn-off delay") + ":",
                    width: 173,
                    style: { marginTop: '5px' }
                },
                {
                    xtype: 'spinnerfield',
                    width: 50,
                    ref: 'seconds',
                    minValue: 0.0,
                    maxValue: 8.4,
                    allowDecimals: true,
                    defaultValue: 0,
                    allowEmpty: false
                },
                {
                    xtype: 'box',
                    width: 5
                },
                {
                    xtype: 'label',
                    text: dSS._("s"),
                    style: { marginTop: '5px' }
                }
            ]
        });

        Ext.apply(this, {
            bubbleEvents: [ 'formsubmitted', 'formloaded', 'formchanged' ],
            padding: 5,
            scope: this,
            items:
            [
                {
                    xtype: 'label',
                    text: dSS._("Blink behaviour for output")
                },
                {
                    xtype: 'box',
                    height: 10
                },
                this.repetitions,
                this.onDelay,
                this.offDelay
            ]
        });

        dSS.devprops.OutputBlinkConfigForm.superclass.constructor.apply(this, arguments);

        this.on('loadcomplete', function() {
                this.onDelay.seconds.on('valid', function() {
                        this.fireEvent('formchanged');
                    },
                    this);

                this.offDelay.seconds.on('valid', function() {
                         this.fireEvent('formchanged');
                    },
                    this);

                this.repetitions.count.on('valid', function() {
                         this.fireEvent('formchanged');
                    },
                    this);

                this.onDelay.seconds.on('spin', function() {
                        this.fireEvent('formchanged');
                    },
                    this);

                this.offDelay.seconds.on('spin', function() {
                         this.fireEvent('formchanged');
                    },
                    this);

                this.repetitions.count.on('spin', function() {
                         this.fireEvent('formchanged');
                    },
                    this);
            },
            this);

        this.on('beforedestroy', function() {
                this.closed = true;
            },
            this);

        this.on('dataloaded', function(data) {
                if (this.closed === true) {
                    return;
                }

                if (data.loader !== dSS.devprops.Loaders.BLINK_CONFIG) {
                    return;
                }

                if (data.json.ok !== true) {
                    this.repetitions.count.markInvalid(dSS._(data.json.message));

                    this.offDelay.seconds.markInvalid(dSS._(data.json.message));
                    this.onDelay.seconds.markInvalid(dSS._(data.json.message));
                    this.fireEvent('formloaded', true);
                    return;
                }

                this.loadedCount = data.json.result.count;
                this.loadedOnDelay = data.json.result.ondelay.toPrecision(4);
                this.loadedOffDelay = data.json.result.offdelay.toPrecision(4);

                this.onDelay.seconds.setValue(this.loadedOnDelay);
                this.offDelay.seconds.setValue(this.loadedOffDelay);
                this.repetitions.count.setValue(this.loadedCount);

                this.fireEvent('formloaded', true);
            },
            this);

        this.on('submit', function(unmaskEl) {
            if (this.getForm().isValid() !== true) {
                Ext.Msg.alert(dSS._("Error"),
                              dSS._("Please correct values in the red " +
                                    "marked fields and try again"),
                              unmaskEl.unmask());
                return;
            }

            var on = this.onDelay.seconds.getValue();
            var off = this.offDelay.seconds.getValue();
            var count = this.repetitions.count.getValue();

            if ((this.loadedOnDelay == on) &&
                (this.loadedOffDelay == off) &&
                (this.loadedCount == count)) {
                this.fireEvent('formsubmitted');
                return;
            }

            // do not submit values that have not changed
            if (this.loadedCount === count) {
                count = '';
            }

            if (this.loadedOnDelay === on) {
                on = '';
            }

            if (this.loadedOffDelay === off) {
                off = '';
            }

            dSS.util.genericRequest.defer(2000, this,
            [
                {
                    url: '/json/device/setBlinkConfig',
                    params:
                    {
                        dsuid: this.dSUID,
                        count: count,
                        ondelay: on,
                        offdelay: off
                    },
                    failureMsg: dSS._("Could not save blink configuration"),
                    scope: this,
                    evt: 'formsubmitted'
                }
            ]);
        },
        this);
    }
});

Ext.reg('outputblinkconfigform', dSS.devprops.OutputBlinkConfigForm);

